#include<iostream>
#include<fstream>
#include<cstdio>
#include<cstdlib>
#include<cstring>
#include<time.h>
#include<unistd.h>
#include<errno.h>
#include<sys/types.h>
#include<sys/wait.h>
#include<sys/socket.h>
#include<netinet/in.h>
#include<arpa/inet.h>
#include<netdb.h>
#include"mysock.h"

/* -------- class mysock -------- */

mysock::mysock(){
    sockfd = -1;
    err = 0;
    errstr = NULL;
}

bool mysock::connected(){
    return (this->sockfd != -1);
}

int mysock::sendmsg(char * buf, int len){
    if(!connected()){
        err = 1;
        errstr = (char *)"Not connected.";
        return -1;
    }
    if (send(sockfd, buf, len, 0) < len){
        err = 1;
        errstr = (char *)"Data Not Send.";
        return -1;
    }
    err = 0;
    return len;
}

int mysock::recvmsg(char * buf, int len){
    int recv_len = -1;
    if(!connected()){
        err = errno;
        errstr = (char *)"Not connected";
        return -1;
    }
    recv_len = recv(sockfd, buf, len, 0);
    if(recv_len == -1){
        err = errno;
        errstr = (char *)"Data Not Received";
        return -1;
    }
    err = 0;
    return recv_len;
}

int mysock::sendline(char * buf){
    int i= 0;
    for (; buf[i] != '\r' && buf[i] != '\n' && buf[i] != '\0'; ++i){
        sendmsg(buf+i, 1);
        if(err) return -1;
    }
    i += sendmsg((char *)"\r\n", 2);
#if DEBUG == 1    
    printf("--> %s", buf);
#endif    
    return i;
}

int mysock::recvline(char * buf){
    int i = 0, recv_len = 0;
    do{
        recv_len = recvmsg(buf+i, 1);
        if(err) return -1;
        if(i >= 1023) break;
    }while(recv_len == 1 && buf[i++] != '\n');
    buf[i] = 0;
    printf("<-- %s", buf);
    return i;
}

bool mysock::quit(){
    return close(sockfd);
}

void mysock::dealerr(){
    if (err){
        perror(errstr);
        close(sockfd);
        exit(errno);
    }
}

/* -------- class mysock_client -------- */

mysock_client::mysock_client(){
    sockfd = -1;
    err = 0;
    errstr = NULL;
}

//给出server和port字符串初始化
mysock_client::mysock_client(char *server_str, char *port_str){
   conn(server_str, port_str);
}

//给出server和port字符串连接
int mysock_client::conn(char *server_str, char *port_str){
    sockfd = socket(PF_INET, SOCK_STREAM, 0);
    if (sockfd == -1){
        err = errno;
        errstr = (char *)"socket";
        return false;
    }
    host.sin_family = AF_INET;
    struct hostent *he;
    if(sscanf(server_str, "%*d.%*d.%*d.%*d") == 4){ //IP
        he = gethostbyaddr(server_str, 4, AF_INET);
    }else{ //Domain Name
        he = gethostbyname(server_str);
    }
    if (he == NULL) {
        err = 1;
        errstr = (char *)"gethostbyaddr";
        return false;
    }
    host.sin_addr = *((struct in_addr*)he->h_addr);
    printf("Server:\t%s(%s)\n", server_str, inet_ntoa(host.sin_addr));

    int port;
    sscanf(port_str, "%d", &port);
    printf("Port:\t%d\n", port);
    host.sin_port = htons(port);

    bzero(&host.sin_zero, 8);

    if (connect(sockfd, (struct sockaddr*)&host, sizeof(host)) == -1){
        err = 1;
        errstr = (char *)"connect";
        return false;
    }

    printf("Connected to server (%s)\n", inet_ntoa(host.sin_addr));
    err = 0;
    return true;
}

/* -------- class mysock_server -------- */

mysock_server::mysock_server(){
    sockfd = -1;
    err = 0;
    errstr = NULL;
}

mysock_server::mysock_server(char *port_str){
    conn(port_str);
}

bool mysock_server::conn(char *port_str){
    int port;
    sscanf(port_str, "%d", &port);

    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1){
        perror("socket");
        return false;
    }

    //初始化主机地址信息
    struct sockaddr_in hostaddr;
    hostaddr.sin_family = AF_INET;
    hostaddr.sin_port = htons(port);
    hostaddr.sin_addr.s_addr = INADDR_ANY;
    bzero((char *) &(hostaddr.sin_zero), 8);

    //绑定
    if(bind(sockfd, (struct sockaddr *) &hostaddr, sizeof(sockaddr)) == -1){
        perror("bind");
        return false;
    }

    //监听端口
    if (listen(sockfd, 1) == -1){
        perror("listen");
        return false;
    }

    //等待客户端连接
    socklen_t sin_size = sizeof(struct sockaddr_in);
    int new_fd;
    struct sockaddr_in clientaddr;
    if((new_fd = accept(sockfd, (struct sockaddr *)&clientaddr, &sin_size)) == -1){
        perror("accept");
        return false;
    }
    //接受到一个请求
    printf("(t=%u)Server: accepted from %s\n", 
            (unsigned int)time(NULL), inet_ntoa(clientaddr.sin_addr));
    while(true){
        ;
    }
    //关闭new_fd
    close(new_fd);
}
